@@ -2,7 +2,9 @@
 
 from __future__ import annotations
 
-from typing import Any
+TYPE_CHECKING = False
+if TYPE_CHECKING:
+    from typing import Any
 
 __all__ = ["natural_list"]
 
