@@ -2,25 +2,27 @@
 
 These are largely borrowed from Django's `contrib.humanize`.
 """
+
 from __future__ import annotations
 
-import collections.abc
-import datetime as dt
-import math
-import typing
 from enum import Enum
 from functools import total_ordering
-from typing import Any
 
 from .i18n import _gettext as _
 from .i18n import _ngettext
 from .number import intcomma
 
+TYPE_CHECKING = False
+if TYPE_CHECKING:
+    import datetime as dt
+    from collections.abc import Iterable
+    from typing import Any
+
 __all__ = [
+    "naturaldate",
+    "naturalday",
     "naturaldelta",
     "naturaltime",
-    "naturalday",
-    "naturaldate",
     "precisedelta",
 ]
 
@@ -36,13 +38,15 @@ class Unit(Enum):
     MONTHS = 6
     YEARS = 7
 
-    def __lt__(self, other: typing.Any) -> typing.Any:
+    def __lt__(self, other: Any) -> Any:
         if self.__class__ is other.__class__:
             return self.value < other.value
         return NotImplemented
 
 
 def _now() -> dt.datetime:
+    import datetime as dt
+
     return dt.datetime.now()
 
 
@@ -62,12 +66,14 @@ def _abs_timedelta(delta: dt.timedelta) -> dt.timedelta:
 
 
 def _date_and_delta(
-    value: typing.Any, *, now: dt.datetime | None = None
-) -> tuple[typing.Any, typing.Any]:
+    value: Any, *, now: dt.datetime | None = None, precise: bool = False
+) -> tuple[Any, Any]:
     """Turn a value into a date and a timedelta which represents how long ago it was.
 
     If that's not possible, return `(None, value)`.
     """
+    import datetime as dt
+
     if not now:
         now = _now()
     if isinstance(value, dt.datetime):
@@ -78,7 +84,7 @@ def _date_and_delta(
         delta = value
     else:
         try:
-            value = int(value)
+            value = value if precise else round(value)
             delta = dt.timedelta(seconds=value)
             date = now - delta
         except (ValueError, TypeError):
@@ -95,6 +101,8 @@ def naturaldelta(
 
     This is similar to `naturaltime`, but does not add tense to the result.
 
+    The timedelta will be rounded to the nearest unit that makes sense.
+
     Args:
         value (datetime.timedelta, int or float): A timedelta or a number of seconds.
         months (bool): If `True`, then a number of months (based on 30.5 days) will be
@@ -104,23 +112,30 @@ def naturaldelta(
     Returns:
         str (str or `value`): A natural representation of the amount of time
             elapsed unless `value` is not datetime.timedelta or cannot be
-            converted to int. In that case, a `value` is returned unchanged.
+            converted to int (cannot be float due to 'inf' or 'nan').
+            In that case, a `value` is returned unchanged.
 
     Raises:
         OverflowError: If `value` is too large to convert to datetime.timedelta.
 
-    Examples
+    Examples:
         Compare two timestamps in a custom local timezone::
 
-        import datetime as dt
-        from dateutil.tz import gettz
+        ```pycon
+        >>> import datetime as dt
+        >>> from dateutil.tz import gettz
 
-        berlin = gettz("Europe/Berlin")
-        now = dt.datetime.now(tz=berlin)
-        later = now + dt.timedelta(minutes=30)
+        >>> berlin = gettz("Europe/Berlin")
+        >>> now = dt.datetime.now(tz=berlin)
+        >>> later = now + dt.timedelta(minutes=30)
+
+        >>> assert naturaldelta(later - now) == "30 minutes"
+        True
+        ```
 
-        assert naturaldelta(later - now) == "30 minutes"
     """
+    import datetime as dt
+
     tmp = Unit[minimum_unit.upper()]
     if tmp not in (Unit.SECONDS, Unit.MILLISECONDS, Unit.MICROSECONDS):
         msg = f"Minimum unit '{minimum_unit}' not supported"
@@ -131,21 +146,21 @@ def naturaldelta(
         delta = value
     else:
         try:
-            value = int(value)
+            int(value)  # Explicitly don't support string such as "NaN" or "inf"
+            value = float(value)
             delta = dt.timedelta(seconds=value)
         except (ValueError, TypeError):
             return str(value)
 
     use_months = months
 
-    seconds = abs(delta.seconds)
-    days = abs(delta.days)
-    years = days // 365
-    days = days % 365
-    num_months = int(days // 30.5)
+    delta = abs(delta)
+    years = delta.days // 365
+    days = delta.days % 365
+    num_months = round(days / 30.5)
 
-    if not years and days < 1:
-        if seconds == 0:
+    if years == 0 and days < 1:
+        if delta.seconds == 0:
             if min_unit == Unit.MICROSECONDS and delta.microseconds < 1000:
                 return (
                     _ngettext("%d microsecond", "%d microseconds", delta.microseconds)
@@ -162,24 +177,30 @@ def naturaldelta(
                 )
             return _("a moment")
 
-        if seconds == 1:
+        if delta.seconds == 1:
             return _("a second")
 
-        if seconds < 60:
-            return _ngettext("%d second", "%d seconds", seconds) % seconds
+        if delta.seconds < 60:
+            return _ngettext("%d second", "%d seconds", delta.seconds) % delta.seconds
 
-        if 60 <= seconds < 120:
-            return _("a minute")
+        if 60 <= delta.seconds < 3600:
+            minutes = round(delta.seconds / 60)
+            if minutes == 1:
+                return _("a minute")
+
+            if minutes == 60:
+                return _("an hour")
 
-        if 120 <= seconds < 3600:
-            minutes = seconds // 60
             return _ngettext("%d minute", "%d minutes", minutes) % minutes
 
-        if 3600 <= seconds < 3600 * 2:
-            return _("an hour")
+        if 3600 <= delta.seconds:
+            hours = round(delta.seconds / 3600)
+            if hours == 1:
+                return _("an hour")
+
+            if hours == 24:
+                return _("a day")
 
-        if 3600 < seconds:
-            hours = seconds // 3600
             return _ngettext("%d hour", "%d hours", hours) % hours
 
     elif years == 0:
@@ -189,25 +210,32 @@ def naturaldelta(
         if not use_months:
             return _ngettext("%d day", "%d days", days) % days
 
-        if not num_months:
+        if num_months == 0:
             return _ngettext("%d day", "%d days", days) % days
 
         if num_months == 1:
             return _("a month")
 
+        if num_months == 12:
+            return _("a year")
+
         return _ngettext("%d month", "%d months", num_months) % num_months
 
     elif years == 1:
-        if not num_months and not days:
+        if num_months == 0 and days == 0:
             return _("a year")
 
-        if not num_months:
+        if num_months == 0:
             return _ngettext("1 year, %d day", "1 year, %d days", days) % days
 
         if use_months:
             if num_months == 1:
                 return _("1 year, 1 month")
 
+            if num_months == 12:
+                years += 1
+                return _ngettext("%d year", "%d years", years) % years
+
             return (
                 _ngettext("1 year, %d month", "1 year, %d months", num_months)
                 % num_months
@@ -229,6 +257,8 @@ def naturaltime(
 
     This is more or less compatible with Django's `naturaltime` filter.
 
+    The time will be rounded to the nearest unit that makes sense.
+
     Args:
         value (datetime.datetime, datetime.timedelta, int or float): A `datetime`, a
             `timedelta`, or a number of seconds.
@@ -244,6 +274,8 @@ def naturaltime(
     Returns:
         str: A natural representation of the input in a resolution that makes sense.
     """
+    import datetime as dt
+
     value = _convert_aware_datetime(value)
     when = _convert_aware_datetime(when)
 
@@ -269,6 +301,8 @@ def _convert_aware_datetime(
     value: dt.datetime | dt.timedelta | float | None,
 ) -> Any:
     """Convert aware datetime to naive datetime and pass through any other type."""
+    import datetime as dt
+
     if isinstance(value, dt.datetime) and value.tzinfo is not None:
         value = dt.datetime.fromtimestamp(value.timestamp())
     return value
@@ -282,6 +316,8 @@ def naturalday(value: dt.date | dt.datetime, format: str = "%b %d") -> str:
     formatted according to `format`.
 
     """
+    import datetime as dt
+
     try:
         value = dt.date(value.year, value.month, value.day)
     except AttributeError:
@@ -306,6 +342,8 @@ def naturalday(value: dt.date | dt.datetime, format: str = "%b %d") -> str:
 
 def naturaldate(value: dt.date | dt.datetime) -> str:
     """Like `naturalday`, but append a year for dates more than ~five months away."""
+    import datetime as dt
+
     try:
         value = dt.date(value.year, value.month, value.day)
     except AttributeError:
@@ -325,81 +363,47 @@ def _quotient_and_remainder(
     divisor: float,
     unit: Unit,
     minimum_unit: Unit,
-    suppress: collections.abc.Iterable[Unit],
+    suppress: Iterable[Unit],
+    format: str,
 ) -> tuple[float, float]:
-    """Divide `value` by `divisor` returning the quotient and remainder.
+    """Divide `value` by `divisor`, returning the quotient and remainder.
 
-    If `unit` is `minimum_unit`, makes the quotient a float number and the remainder
-    will be zero. The rational is that if `unit` is the unit of the quotient, we cannot
-    represent the remainder because it would require a unit smaller than the
-    `minimum_unit`.
+    If `unit` is `minimum_unit`, the quotient will be the rounding of `value / divisor`
+    according to the `format` string and the remainder will be zero. The rationale is
+    that if `unit` is the unit of the quotient, we cannot represent the remainder
+    because it would require a unit smaller than the `minimum_unit`.
 
     >>> from humanize.time import _quotient_and_remainder, Unit
-    >>> _quotient_and_remainder(36, 24, Unit.DAYS, Unit.DAYS, [])
+    >>> _quotient_and_remainder(36, 24, Unit.DAYS, Unit.DAYS, [], "%0.2f")
     (1.5, 0)
 
-    If unit is in `suppress`, the quotient will be zero and the remainder will be the
+    If `unit` is in `suppress`, the quotient will be zero and the remainder will be the
     initial value. The idea is that if we cannot use `unit`, we are forced to use a
-    lower unit so we cannot do the division.
+    lower unit, so we cannot do the division.
 
-    >>> _quotient_and_remainder(36, 24, Unit.DAYS, Unit.HOURS, [Unit.DAYS])
+    >>> _quotient_and_remainder(36, 24, Unit.DAYS, Unit.HOURS, [Unit.DAYS], "%0.2f")
     (0, 36)
 
-    In other case return quotient and remainder as `divmod` would do it.
+    In other cases, return the quotient and remainder as `divmod` would do it.
 
-    >>> _quotient_and_remainder(36, 24, Unit.DAYS, Unit.HOURS, [])
+    >>> _quotient_and_remainder(36, 24, Unit.DAYS, Unit.HOURS, [], "%0.2f")
     (1, 12)
 
     """
     if unit == minimum_unit:
-        return value / divisor, 0
+        return _rounding_by_fmt(format, value / divisor), 0
 
     if unit in suppress:
         return 0, value
 
-    return divmod(value, divisor)
+    # Convert the remainder back to integer is necessary for months. 1 month is 30.5
+    # days on average, but if we have 31 days, we want to count is as a whole month,
+    # and not as 1 month plus a remainder of 0.5 days.
+    q, r = divmod(value, divisor)
+    return q, int(r)
 
 
-def _carry(
-    value1: float,
-    value2: float,
-    ratio: float,
-    unit: Unit,
-    min_unit: Unit,
-    suppress: typing.Iterable[Unit],
-) -> tuple[float, float]:
-    """Return a tuple with two values.
-
-    If the unit is in `suppress`, multiply `value1` by `ratio` and add it to `value2`
-    (carry to right). The idea is that if we cannot represent `value1` we need to
-    represent it in a lower unit.
-
-    >>> from humanize.time import _carry, Unit
-    >>> _carry(2, 6, 24, Unit.DAYS, Unit.SECONDS, [Unit.DAYS])
-    (0, 54)
-
-    If the unit is the minimum unit, `value2` is divided by `ratio` and added to
-    `value1` (carry to left). We assume that `value2` has a lower unit so we need to
-    carry it to `value1`.
-
-    >>> _carry(2, 6, 24, Unit.DAYS, Unit.DAYS, [])
-    (2.25, 0)
-
-    Otherwise, just return the same input:
-
-    >>> _carry(2, 6, 24, Unit.DAYS, Unit.SECONDS, [])
-    (2, 6)
-    """
-    if unit == min_unit:
-        return value1 + value2 / ratio, 0
-
-    if unit in suppress:
-        return 0, value2 + value1 * ratio
-
-    return value1, value2
-
-
-def _suitable_minimum_unit(min_unit: Unit, suppress: typing.Iterable[Unit]) -> Unit:
+def _suitable_minimum_unit(min_unit: Unit, suppress: Iterable[Unit]) -> Unit:
     """Return a minimum unit suitable that is not suppressed.
 
     If not suppressed, return the same unit:
@@ -428,7 +432,7 @@ def _suitable_minimum_unit(min_unit: Unit, suppress: typing.Iterable[Unit]) -> U
     return min_unit
 
 
-def _suppress_lower_units(min_unit: Unit, suppress: typing.Iterable[Unit]) -> set[Unit]:
+def _suppress_lower_units(min_unit: Unit, suppress: Iterable[Unit]) -> set[Unit]:
     """Extend suppressed units (if any) with all units lower than the minimum unit.
 
     >>> from humanize.time import _suppress_lower_units, Unit
@@ -445,12 +449,12 @@ def _suppress_lower_units(min_unit: Unit, suppress: typing.Iterable[Unit]) -> se
 
 
 def precisedelta(
-    value: dt.timedelta | int | None,
+    value: dt.timedelta | float | None,
     minimum_unit: str = "seconds",
-    suppress: typing.Iterable[str] = (),
+    suppress: Iterable[str] = (),
     format: str = "%0.2f",
 ) -> str:
-    """Return a precise representation of a timedelta.
+    """Return a precise representation of a timedelta or number of seconds.
 
     ```pycon
     >>> import datetime as dt
@@ -516,14 +520,14 @@ def precisedelta(
 
     ```
     """
-    date, delta = _date_and_delta(value)
+    date, delta = _date_and_delta(value, precise=True)
     if date is None:
         return str(value)
 
     suppress_set = {Unit[s.upper()] for s in suppress}
 
-    # Find a suitable minimum unit (it can be greater the one that the
-    # user gave us if it is suppressed).
+    # Find a suitable minimum unit (it can be greater than the one that the
+    # user gave us, if that one is suppressed).
     min_unit = Unit[minimum_unit.upper()]
     min_unit = _suitable_minimum_unit(min_unit, suppress_set)
     del minimum_unit
@@ -553,27 +557,57 @@ def precisedelta(
     #       years, days = divmod(years, days)
     #
     # The same applies for months, hours, minutes and milliseconds below
-    years, days = _quotient_and_remainder(days, 365, YEARS, min_unit, suppress_set)
-    months, days = _quotient_and_remainder(days, 30.5, MONTHS, min_unit, suppress_set)
-
-    # If DAYS is not in suppress, we can represent the days but
-    # if it is a suppressed unit, we need to carry it to a lower unit,
-    # seconds in this case.
-    #
-    # The same applies for secs and usecs below
-    days, secs = _carry(days, secs, 24 * 3600, DAYS, min_unit, suppress_set)
-
-    hours, secs = _quotient_and_remainder(secs, 3600, HOURS, min_unit, suppress_set)
-    minutes, secs = _quotient_and_remainder(secs, 60, MINUTES, min_unit, suppress_set)
-
-    secs, usecs = _carry(secs, usecs, 1e6, SECONDS, min_unit, suppress_set)
-
-    msecs, usecs = _quotient_and_remainder(
-        usecs, 1000, MILLISECONDS, min_unit, suppress_set
+    years, days = _quotient_and_remainder(
+        days, 365, YEARS, min_unit, suppress_set, format
+    )
+    months, days = _quotient_and_remainder(
+        days, 30.5, MONTHS, min_unit, suppress_set, format
     )
 
-    # if _unused != 0 we had lost some precision
-    usecs, _unused = _carry(usecs, 0, 1, MICROSECONDS, min_unit, suppress_set)
+    secs = days * 24 * 3600 + secs
+    days, secs = _quotient_and_remainder(
+        secs, 24 * 3600, DAYS, min_unit, suppress_set, format
+    )
+
+    hours, secs = _quotient_and_remainder(
+        secs, 3600, HOURS, min_unit, suppress_set, format
+    )
+    minutes, secs = _quotient_and_remainder(
+        secs, 60, MINUTES, min_unit, suppress_set, format
+    )
+
+    usecs = secs * 1e6 + usecs
+    secs, usecs = _quotient_and_remainder(
+        usecs, 1e6, SECONDS, min_unit, suppress_set, format
+    )
+
+    msecs, usecs = _quotient_and_remainder(
+        usecs, 1000, MILLISECONDS, min_unit, suppress_set, format
+    )
+
+    # Due to rounding, it could be that a unit is high enough to be promoted to a higher
+    # unit. Example: 59.9 minutes was rounded to 60 minutes, and thus it should become 0
+    # minutes and one hour more.
+    if msecs >= 1_000 and SECONDS not in suppress_set:
+        msecs -= 1_000
+        secs += 1
+    if secs >= 60 and MINUTES not in suppress_set:
+        secs -= 60
+        minutes += 1
+    if minutes >= 60 and HOURS not in suppress_set:
+        minutes -= 60
+        hours += 1
+    if hours >= 24 and DAYS not in suppress_set:
+        hours -= 24
+        days += 1
+    # When adjusting we should not deal anymore with fractional days as all rounding has
+    # been already made. We promote 31 days to an extra month.
+    if days >= 31 and MONTHS not in suppress_set:
+        days -= 31
+        months += 1
+    if months >= 12 and YEARS not in suppress_set:
+        months -= 12
+        years += 1
 
     fmts = [
         ("%d year", "%d years", years),
@@ -592,9 +626,13 @@ def precisedelta(
         if fmt_value > 0 or (not texts and unit == min_unit):
             _fmt_value = 2 if 1 < fmt_value < 2 else int(fmt_value)
             fmt_txt = _ngettext(singular_txt, plural_txt, _fmt_value)
+            import math
+
             if unit == min_unit and math.modf(fmt_value)[0] > 0:
                 fmt_txt = fmt_txt.replace("%d", format)
             elif unit == YEARS:
+                if math.modf(fmt_value)[0] == 0:
+                    fmt_value = int(fmt_value)
                 fmt_txt = fmt_txt.replace("%d", "%s")
                 texts.append(fmt_txt % intcomma(fmt_value))
                 continue
@@ -611,3 +649,24 @@ def precisedelta(
     tail = texts[-1]
 
     return _("%s and %s") % (head, tail)
+
+
+def _rounding_by_fmt(format: str, value: float) -> float | int:
+    """Round a number according to the string format provided.
+
+    The string format is the old printf-style string formatting.
+
+    If we are using a format which truncates the value, such as "%d" or "%i", the
+    returned value will be of type `int`.
+
+    If we are using a format which rounds the value, such as "%.2f" or even "%.0f",
+    we will return a float.
+    """
+    result = format % value
+
+    try:
+        value = int(result)
+    except ValueError:
+        value = float(result)
+
+    return value
